const { forEachObjIndexed, omit } = require('ramda');

const MS = require('./methods');

const modelScheme = (origin, scheme) => {
    // recorre origin y aplica la scheme
    const model = scheme(MS);
    let result = {};
    
    forEachObjIndexed((func, key) => {
        result = { ...result, ...func(key, origin) }
    }, model);

    return {...result};
}

module.exports = modelScheme;

// TEST

const originalPerson = {
    name: 'Harley Espinoza',
    age: '24',
    telephone: 85806812,
    address: {
        house: 'Belen, Heredia',
        girl: 'San Rafael, Heredia',
        home: 'Golfito, Puntarenas'
    },
    interests: ['videogames', 'soccer', 'technology']
}

const personScheme = (MS) => ({
    name: MS.rename('fullname'),
    age: MS.defaultType('number'),
    address: MS.picks(['house', 'home']),
    birthday: MS.enhacer(({ age }) => 2019 - age),
    telephone: MS.custom({ defaultType: 'string', rename: 'phone' }),
    interests: MS.omit([1, 2])
});


 const personModel = modelScheme(originalPerson, personScheme);

 const personContact = omit(['age', 'birhday', 'interests'], personModel);

 console.log(personModel);

 /*
{
    fullname: 'Harley Espinoza',
    age: 18,
    phone: '85806812',
    birthday: 1995,
    address: { house:'Belen, Heredia', home: 'Golfito, Puntarenas' },
    interests: 'videogames'
}

{
    fullname: 'Harley Espinoza',
    phone: '85806812',
    address: { house:'Belen, Heredia', home: 'Golfito, Puntarenas' },
}
*/