const { forEachObjIndexed, pathOr, includes, pickAll, omit: omitR } = require('ramda');

const U = require('./utils');

//  cambia el nombre de un atributo
const rename = (name) => (key, origin) => {
    let result = { ...origin};
    if (U.isValidValue(name) && U.isString(name)) {
    //  crea nuevo atributo con el valor de la key
        const value = pathOr(undefined, [key], origin);
        result = { ...result, [name]: value };
        // elimina el viejo atributo
        if (value) {
            delete result[key];
        }
    }
    return result;
}

// define un tipo para un atributo
const defaultType = (type) => (key, origin) => {
    let result = {};
    const typeOf = U.oneOfTypes(type);
    if (typeOf) {
        // si existe value ? covertir al tipo sino asignar default segun tipo
        const value = pathOr(undefined, [key], origin);
        result = { [key]: U.isValidValue(value) ? U.parseValue(typeOf, value) : U.defaultTo(typeOf) }
    }
    return result;
}

// obtiene los items definidos de una collecion
const picks = (collection) => (key, origin) => {
    let result = {};
    if (U.isValidValue(collection) && U.isArray(collection)) {
        const value = pathOr(undefined, [key], origin);
        // filtrar segun tipo
        if (U.isArray(value)) {
            result = { [key]: value.filter((_, i) => includes(i, collection))  }
        }

        if (U.isObject(value)) {
            result = { [key]: pickAll(collection, value) }
        }
    }
    return result;
}

// obtiene los items no definidos de una collecion
const omit = (collection) => (key, origin) => {
    let result = {};
    if (U.isValidValue(collection) && U.isArray(collection)) {
        const value = pathOr(undefined, [key], origin);
        // filtrar segun tipo
        if (U.isArray(value)) {
            result = { [key]: value.filter((_, i) => !includes(i, collection))  }
        }

        if (U.isObject(value)) {
            result = { [key]: omitR(collection, value) }
        }
    }
    return result;
}

// accion sobre uno o mas atributos del objeto original
const enhacer = (action) => (key, origin) => {
    let result = {};
    if (U.isValidValue(action) && U.isFunction(action)) {
        result = { [key]: action(origin) }
    }
    return result;
}

// todas las anteriores juntas
const custom = (config) => (key, origin) => {
    // let result = {...origin};
    // if (U.isValidValue(config) && U.isObject(config)) {
    //     const kit = { rename, defaultType, picks, omit };
    //     forEachObjIndexed((entry, keyFunc) => {
    //         const func = pathOr(undefined, [keyFunc], kit);
    //         if (func) {
    //             result = { ...result, ...func(entry)(key, origin) }
    //         }
    //     }, config);
    // }

    // return result;
    return {};
}

module.exports = { rename, defaultType, custom, picks, omit, enhacer };