// tool functions
const { includes, toLower, type, isNil, split, either, pathOr, isEmpty, complement, mergeRight, forEach, forEachObjIndexed, values } = require('ramda');

// output
const print = console.warn;

// validadores
const isValidValue = complement(either(isNil, isEmpty));
const isArray = entry => toLower(type(entry)) === 'array';
const isString = entry => toLower(type(entry)) === 'string';
const isNumber = entry => toLower(type(entry)) === 'number';
const isObject = entry => toLower(type(entry)) === 'object';
const isFunction = entry => toLower(type(entry)) === 'function';
const oneOfTypes = entry => {
	const typeOf = toLower(entry);
	if (includes(typeOf, ['array', 'string', 'number', 'object', 'function'])) {
		return typeOf;
	}
	return undefined;
}

const parseValue = (entry, value) => {
	switch (entry) {
		case 'array': return isArray(value) ? value : [value];
		case 'string': return isString(value) ? value : String(value);
		case 'number': return isNumber(value) ? value : Number(value);
		case 'object': return isObject(value) ? value : { value };
		default: return isFunction(value) ? value : () => value
	}
}

const defaultTo = (entry) => {
	switch (entry) {
		case 'array': return [];
		case 'string': return '';
		case 'number': return 0;
		case 'object': return {};
		default: return () => {};
	}
}

// to assign new sets values
const assign = (attributeKey, nAssignment, oldObject) => ({ ...oldObject[attributeKey], ...nAssignment });

const mapper = (collection, oldObject) => {
    let result = { ...oldObject};
    forEachObjIndexed((value, key) => {
        result = { ...result, [key]: isFunction(value) ? value(result) : value };
    }, collection);
    return result;
}

const clearAssigment = (stricModel, described, oldModel) => {
    let result = oldModel;
    forEach(val => {
        const key = isArray(val) ? val[0] : val;
        delete oldModel[key];
    }, values(described));

    return ({ ...stricModel, ...result });
}
    

// @param [entry]: Path en dotCase.
// return Path en formato de array.
const getPath = entry => split('.', entry);

// @param [value]: Valor esperado.
// @param [origin]: Objeto original.
// @return Valor obtenido del path o un valor de remplazo.
const neddyValue = (value, origin) => {
	if (!isValidValue(value)) return undefined;
	if (isArray(value)) {
		const [expectedValue, replacement, enhancer] = value;
		const result = pathOr(replacement, getPath(expectedValue), origin);
		if (isValidValue(enhancer) && isFunction(enhancer))
			return enhancer(result);
		return result;
	}
	return pathOr(undefined, getPath(value), origin);
};

// @param [origin]: objeto con toda la data original
// @param [expected]: Objeto que necesito y como lo necesito.
// @param [namespace]: Nombre identificador para debug.
// @return Objeto deseado
const neddyObj = (origin, expected) => {
	if (isObject(origin) && isObject(expected)) {
		if (!isEmpty(origin) && !isEmpty(expected)) {
			let neddy = {};
			forEachObjIndexed((value, key) => {
				// mapea el objeto como array
				neddy = mergeRight(neddy, { [key]: neddyValue(value, origin) });
			}, expected);
			return neddy;
		}
	}
	return {};
};

module.exports = {
    assign,
    mapper,
	getPath,
    neddyObj,
	isArray,
    isObject,
    isFunction,
	isString,
	isNumber,
	oneOfTypes,
	parseValue,
	defaultTo,
	isValidValue,
    clearAssigment
}
